import httpx
import time


async def load_orgs_and_teams(config, profile, access_token):
    store_timestamp = False
    extras = {}
    if config.get("load_orgs"):
        load_orgs = config["load_orgs"]
        gh_orgs = []
        for org in force_list(load_orgs):
            url = "https://api.github.com/orgs/{}/memberships/{}".format(
                org, profile["login"]
            )
            async with httpx.AsyncClient() as client:
                response = await client.get(
                    url, headers={"Authorization": "token {}".format(access_token)}
                )
            if response.status_code == 200 and response.json()["state"] == "active":
                gh_orgs.append(org)
        extras["gh_orgs"] = gh_orgs
        store_timestamp = True
    if config.get("load_teams"):
        load_teams = config["load_teams"]
        gh_teams = []
        for team in force_list(load_teams):
            org_slug, _, team_slug = team.partition("/")
            # Figure out the team_id
            lookup_url = "https://api.github.com/orgs/{}/teams/{}".format(
                org_slug, team_slug
            )
            async with httpx.AsyncClient() as client:
                response = await client.get(
                    lookup_url,
                    headers={"Authorization": "token {}".format(access_token)},
                )
            if response.status_code == 200:
                team_id = response.json()["id"]
            else:
                continue
            # Now check if user is an active member of the team:
            team_membership_url = (
                "https://api.github.com/teams/{}/memberships/{}".format(
                    team_id, profile["login"]
                )
            )
            async with httpx.AsyncClient() as client:
                response = await client.get(
                    team_membership_url,
                    headers={"Authorization": "token {}".format(access_token)},
                )
            if response.status_code == 200 and response.json()["state"] == "active":
                gh_teams.append(team)
        extras["gh_teams"] = gh_teams
        store_timestamp = True

    if store_timestamp:
        extras["gh_ts"] = int(time.time())

    return extras


def force_list(value):
    if isinstance(value, str):
        return [value]
    return value



from datasette.utils.asgi import Response
from urllib.parse import parse_qsl
from .utils import load_orgs_and_teams
import httpx


DEPRECATED_KEYS = ("allow_users", "allow_orgs", "allow_teams")


def verify_config(config):
    config = config or {}
    for key in DEPRECATED_KEYS:
        assert key not in config, "{} is no longer a supported option".format(key)


async def github_auth_start(datasette):
    config = datasette.plugin_config("datasette-auth-github")
    verify_config(config)
    if config.get("load_teams"):
        scope = "read:org"
    elif config.get("load_orgs"):
        scope = "user"
    else:
        scope = "user:email"
    github_login_url = (
        "https://github.com/login/oauth/authorize?scope={}&client_id={}".format(
            scope, config["client_id"]
        )
    )
    return Response.redirect(github_login_url)


async def response_error(datasette, error):
    return Response.html(
        await datasette.render_template(
            "datasette_auth_github_error.html", {"error": error}
        ),
        status=500,
    )


async def github_auth_callback(datasette, request, scope, receive, send):
    config = datasette.plugin_config("datasette-auth-github")
    verify_config(config)
    if not request.args.get("code"):
        return await response_error(datasette, "Authentication failed, no code")

    # Exchange that code for a token
    async with httpx.AsyncClient() as client:
        github_response = await client.post(
            "https://github.com/login/oauth/access_token",
            data={
                "client_id": config["client_id"],
                "client_secret": config["client_secret"],
                "code": request.args["code"],
            },
        )
    parsed = dict(parse_qsl(github_response.text))
    # b'error=bad_verification_code&error_description=The+code+passed...'
    if parsed.get("error"):
        return await response_error(
            datasette, parsed["error"] + ": " + (parsed.get("error_description") or "")
        )
    access_token = parsed.get("access_token")
    if not access_token:
        return await response_error(datasette, "No valid access token")

    # Use access_token to verify user
    profile_url = "https://api.github.com/user"
    try:
        async with httpx.AsyncClient() as client:
            profile = (
                await client.get(
                    profile_url,
                    headers={"Authorization": "token {}".format(access_token)},
                )
            ).json()
    except ValueError:
        return await response_error(datasette, "Could not load GitHub profile")
    actor = {
        "display": profile["login"],
        "gh_id": str(profile["id"]),
        "gh_name": profile["name"],
        "gh_login": profile["login"],
        "gh_email": profile["email"],
    }
    extras = await load_orgs_and_teams(config, profile, access_token)
    actor.update(extras)

    # Set a signed cookie and redirect to homepage
    response = Response.redirect("/")
    response.set_cookie("ds_actor", datasette.sign({"a": actor}, "actor"))
    return response



from datasette import hookimpl
from .views import github_auth_start, github_auth_callback


@hookimpl
def register_routes():
    return [
        (r"^/-/github-auth-start$", github_auth_start),
        (r"^/-/github-auth-callback$", github_auth_callback),
    ]


@hookimpl
def menu_links(datasette, actor):
    if not actor:
        return [
            {
                "href": datasette.urls.path("/-/github-auth-start"),
                "label": "Sign in with GitHub",
            },
        ]



from setuptools import setup
import os

VERSION = "0.13"


def get_long_description():
    with open(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), "README.md"),
        encoding="utf8",
    ) as fp:
        return fp.read()


setup(
    name="datasette-auth-github",
    description="Datasette plugin and ASGI middleware that authenticates users against GitHub",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    author="Simon Willison",
    url="https://github.com/simonw/datasette-auth-github",
    license="Apache License, Version 2.0",
    version=VERSION,
    packages=["datasette_auth_github"],
    entry_points={"datasette": ["auth_github = datasette_auth_github"]},
    install_requires=["datasette>=0.51"],
    extras_require={
        "test": [
            "datasette",
            "pytest",
            "pytest-asyncio",
            "sqlite-utils",
            "pytest-httpx",
        ]
    },
    tests_require=["datasette-auth-github[test]"],
    package_data={"datasette_auth_github": ["templates/*.html"]},
)
